/*
Copyright (c) 2015  V.Ayllón.  All rights reserved.
 
 TwinTeethMC is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 to anyone for the consequences of using it or for whether it serves any
 particular purpose or works at all, unless he says so in writing.  Refer
 to the GNU General Public License, Version 2 or later, for full details.
 
 Everyone is granted permission to copy, modify and redistribute
 TwinTeethMC, but only under the conditions described in the GNU General Public
 License.  A copy of this license is supposed to have been
 given to you along with TwinTeethMC so you can know your rights and
 responsibilities.  It should be in a file named COPYING.  Among other
 things, the copyright notice and this notice must be preserved on all
 copies.
 
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
import controlP5.*;

/**
 ** LoadFile()
 **/
public void LoadFile(int v) {
  if (v == 1) {
    if (connectStatus && printState == 0) {
      loadPath = null;
      if (activeTool == LASER_TOOL)
        selectInput("Select a PostScript or BMP file", "fileSelected");  // Opens file chooser
      else
        selectInput("Select a G-Code or HPGL file", "fileSelected");  // Opens file chooser
      autoState = WAITING_FILE_STATE;
    }
  }
}
/**
 ** fileSelected() - Callback fdr selectInput
 **/
void fileSelected(File selection) {
  if (selection == null) {
    loadPath = null;
    autoState = PRINTER_READY_STATE;  // Load image
  } else {
    loadPath = selection.getAbsolutePath();
  }
}
/**
 ** reopenFile()
 **/
boolean reopenFile() {
  return openFile();
}

/**
 ** openFile()
 **/
boolean openFile() {
  // If a file was selected, print path to file
  if (loadPath != null) {
    if (fileRow !=  null)
      closeFile();
    try {
      // Check if HPGL file and convert it to g-code
      //
      if (loadPath.endsWith(".hp")) {
        fileRow = loadStrings(hpgl.hpgl2Gcode(loadPath));
      } else if (loadPath.endsWith(".gcode") || loadPath.endsWith(".gc")) {
        fileRow = loadStrings(loadPath);
      } else {
        throw new IOException("Bad file extension. Only .hp, .gcode or .gc accepted");
      }
    } 
    catch (Exception e) {
      fileStatus = false;
      printlnMsg("Error: Bad file format. Select a valid HPGL or G-code file.");
      System.out.println("Error : " + e.getMessage());
      return true;
    }

    // File was opened OK

    fileStatus = true;
    printCtrl.setLabel("Run");
    printCtrl.unlock();
    pauseCtrl.setLabel("Pause");
    pauseCtrl.lock();
    printlnMsg("File " + loadPath + " opened: "+ fileRow.length + " rows");
    return false;
  }
  fileStatus = false;
  printlnMsg("Error opening file.");
  return true;
}

/**
 ** openPostScriptFile()
 **/
boolean openPostScriptFile() {
  if (loadPath != null) {
    try {
      if (loadPath.endsWith(".ps")) {
        PostScriptToBMP(loadPath, DEF_MATERIAL_WIDTH, DEF_MATERIAL_HEIGHT); 
        imageFilename = PSTEMPFILENAME;
      } else if (loadPath.endsWith(".bmp")) {
        imageFilename = loadPath;
      } else {
        throw new IOException("Bad file extension. Only BMP or PS accepted.");
      }
      // Load the file
      if (raster.load(imageFilename)) {
        fileStatus = false;
        return true;
      }
    } 
    catch (Exception e) {
      fileStatus = false;
      printlnMsg("Error: Bad file format. Select a valid BMP or PostScript file.");
      System.out.println("PostScriptToImage Error : " + e.getMessage());
      return true;
    }
    // OK File loaded
    fileStatus = true;
    printCtrl.setLabel("Run");
    printCtrl.unlock();
    pauseCtrl.setLabel("Pause");
    pauseCtrl.lock();
    return false;
  }
  fileStatus = false;
  printlnMsg("Error opening file.");
  return true;
}

/**
 ** closeFile()
 **/
void closeFile() {
  //  fileReader = null;
  fileRow = null;
  printCtrl.setLabel("Run");
  printCtrl.lock();
  pauseCtrl.setLabel("Pause");
  pauseCtrl.lock();
}

