/*
Copyright (c) 2015  V.Ayllón.  All rights reserved.
 
 TwinTeethMC is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 to anyone for the consequences of using it or for whether it serves any
 particular purpose or works at all, unless he says so in writing.  Refer
 to the GNU General Public License, Version 2 or later, for full details.
 
 Everyone is granted permission to copy, modify and redistribute
 TwinTeethMC, but only under the conditions described in the GNU General Public
 License.  A copy of this license is supposed to have been
 given to you along with TwinTeethMC so you can know your rights and
 responsibilities.  It should be in a file named COPYING.  Among other
 things, the copyright notice and this notice must be preserved on all
 copies.
 
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

ControlGroup messageBox;
int messageBoxResult = -1;
String messageBoxString = "";

String G54 = "G54 P";
String G93 = "G93";
String G28 = "G28";
String G29 = "G29";
String G31 = "G31";  // Focus laser
String G32 = "G32";  // Init rastering
String G90 = "G90";  // Absolute coordinates
String G91 = "G91";  // Relative coordinates
String G92 = "G92";  // Relative coordinates
String G94 = "G94";
String G95 = "G95";  // Set z min
String G0 =  "G0";
String G7 =  "G7";  // Raster command
String M03 = "M03"; // Set laser power
String M05 = "M05"; // Select diode and laser off
String M06 = "M06"; // Laser on
String M07 = "M07"; // Test mode
String M104 = "M104"; // Set extruder target temperature
String M105 = "M105"; // Read current temperature
String M403 = "M403";
String M508 = "M508"; // Set seek rate


// function buttonA will receive changes from 
// controller with name sendGcode
public void SendGcodeButton(int v) {
  sendCommand(cmdLine.getText().toUpperCase());
  printlnEcho(cmdLine.getText().toUpperCase() +" sent\n");
}

/**
 ** CommandLine()  - automatically receives results from controller Command
 **/
public void CommandLine(String text) {
  sendCommand(cmdLine.getText().toUpperCase());
  printlnEcho(cmdLine.getText().toUpperCase() +" sent\n");
}

/**
 ** SetToolZero() 
 **/
public void SetToolZero(int v) {
  jogCtrl.setArrayValue(new float[] {
    WJOGX_CENTER_POS, WJOGY_CENTER_POS
  }
  );
  if (zMinCtrl[activeTool].getValue() != 0)
    sendCommand(G93+" Z"+String.format("%2.2f", zMinCtrl[activeTool].getValue()));
  else
    sendCommand(G93);
}

/**
 ** AutoBedLevel() 
 **/
public void AutoBedLevel(int v) {
  sendCommand(G29);
}

/**
 **ResetTool () - Reset tool config
 **/
public void ResetTool(int v) {
  sendCommand(G94 + " P" + activeTool);
}


/**
 **HomeAxis () - Move the printer to the home position
 **/
public void HomeAxis(int v) {
  jogCtrl.setArrayValue(new float[] {
    WJOGX_CENTER_POS, WJOGY_CENTER_POS
  }
  );
  sendCommand(G28); // use absolute coordinates
}


/**
 ** GotoZero() - Move printer to 0,0,0
 **/
void GotoZero(int v) {
  jogCtrl.setArrayValue(new float[] {
    WJOGX_CENTER_POS, WJOGY_CENTER_POS
  }
  );
  absMove(0, 0, 0, (int)xyFeedCtrl.getValue());
}


/**
 ** centerXY() - Move printer to 0,0
 **/
void CenterXY(int v) {
  jogCtrl.setArrayValue(new float[] {
    WJOGX_CENTER_POS, WJOGY_CENTER_POS
  }
  );
  absMove(0, 0, (int)xyFeedCtrl.getValue());
}
/**
 **Zzero () - Move the printer to the Z zero position
 **/
public void ZetaZero(int v) {
  sendCommand(G90);
  sendCommand("G0 Z0 F"+(int)zFeedCtrl.getValue());
}
/**
 ** setDefSeekRate() - Set default seek rate
 **/
public void setDefSeekRate(int seekRate) {
  if (printerStatus)
    sendCommand(M508+" F" + String.format("%d", seekRate));
}
/**
 ** setZMin() - Set the Z axis min position
 **/
public void setZMin(float value) {
  if (printerStatus)
    sendCommand(G95+" Z" + String.format("%2.2f", value));
}
/**
 **relativeMove () - Move the printer some relative positions
 **/
public void relativeMove(float x, float y, float z, int feedrate) {
  String s = G0;
  if (x != 0)
    s += " X" + x;
  if (y != 0)
    s += " Y" + y;
  if (z != 0)
    s += " Z" + z;
  sendCommand(G91);
  sendCommand(s+" F"+feedrate);
}

/**
 **absMove () - Move the printer to an absolute positions in x,y,z
 **/
public void absMove(float x, float y, float z, int feedrate) {
  String s = G0;
  s += " X" + String.format("%2.2f", x);
  s += " Y" + String.format("%2.2f", y);
  s += " Z" + String.format("%3.2f", z);
  sendCommand(G90);
  sendCommand(s+" F"+String.format("%d", feedrate));
}

/**
 **absMove () - Move the printer to an absolute positions in x, y
 **/
public void absMove(float x, float y, int feedrate) {
  String s = G0;
  s += " X" + String.format("%2.2f", x);
  s += " Y" + String.format("%2.2f", y);
  sendCommand(G90);
  sendCommand(s+" F"+String.format("%d", feedrate));
}
/**
 **genG7Gcode() - Generates gcode for raster G7 command
 **/
public String genG7Gcode(String data, int feedRate, int laserPower, int len, int dir) {
  return G7+" F"+feedRate+" P"+laserPower+" L"+len+" N"+dir+" D"+data.substring(0, len);
}

/**
 ** SetLaserDiode() - Select the laser diode
 **/
public void SetLaserDiode(int d) {
  if (printerStatus) {
    if (d <= 0) return;
    sendCommand(M05+" D" + d);
    // Check if laser is on
    // if so re-trurn it on
    if (int(laserOnCtrl.getItem(0).getValue()) > 0)
      turnLaserOn();
  }
}
/**
 ** setFocusDiode() - Select the default focus diode
 **/
public void setFocusDiode(int d) {
  if (d <= 0) return;  
  sendCommand(M05+" F" + d);
}

/**
 ** Focus() - Send focus command and set dimensions in points
 **/
public void Focus() {
  if (printerStatus) {
    laserOnCtrl.getItem(0).setValue(0);
     setFocusDiode(DEFAULT_FOCUS_DIODE);
    if (raster.isLoaded() && raster.getWidth() > 0 && raster.getHeight() > 0) {
      sendCommand(G31+" O" + String.format("%1.1f", focusOffsetCtrl.getValue()) + " W" + raster.getWidth() +" H" + raster.getHeight());
    } else {
      printlnMsg("Error: Please, load a PCB image file before try focus printer");
    }
  }
}
/**
 ** FocusOne() - Send focus one time command
 **/
public void FocusOne() {
  if (printerStatus) {
    laserOnCtrl.getItem(0).setValue(0);
    setFocusDiode(DEFAULT_FOCUS_DIODE);
    sendCommand(G31+" T");
  }
}
/**
 ** sendInitRasterCmd() - Send focus command and set dimensions in points
 **/
public void sendInitRasterCmd(int d, int w, int h) {
  if (printerStatus) {
    sendCommand(G32+" D" + d + " W" + w +" H" + h);
  }
}
/**
 ** endPrinting() - End printing
 **/
void endPrinting() {
  turnLaserOff(); // Spindle or Laser OFF
  relativeMove(0, 0, +10, (int)zFeedCtrl.getValue());
  absMove(0, 0, (int)xyFeedCtrl.getValue());  // Move to 0,0,0
}

/**
 ** turnLaserOn() - Turn laser on
 **/
void turnLaserOn() {
  if (printerStatus) {
    setLaserPower(defLaserPower);
    sendCommand(M06);
  }
}
/**
 ** turnLaserOff() - Turn laser off
 **/
void turnLaserOff() {
  sendCommand(M05+" D0");  // Spindle or Laser OFF
}
/**
 ** setLaserPower() - Set the laser power
 **/
void setLaserPower(int p) {
  sendCommand(M03+" S" + p);
}
/**
 ** setTargetTemp() - Set extruder target temperature
 **/
void setTargetTemp(int t) {
  sendCommand(M104+" S" + t);
}
/**
 ** readCurrentTemp() - read current temperature
 **/
void readCurrentTemp() {
  sendCommand(M105);
}
/**
 ** extrudeFilament() - Extrude some filament
 **/
void extrudeFilament(int len, int feedrate) {
  sendCommand(G92+" E0"); //Reset extruder
  sendCommand(G91); //Relative coordinates
  sendCommand(G0+" E" + len + " F"+feedrate); //Move extruder
}
/**
 ** dispensePaste() - Dispense some paste
 **/
void dispensePaste(float len, int feedrate) {
  sendCommand(G92+" E0"); //Reset dispenser
  sendCommand(G91); //Relative coordinates
  sendCommand(G0+" E" + String.format("%2.2f", len) + " F"+feedrate); //Move dispenser
}
/**
 ** setTestMode() - Set test mode on/off
 **/
void setTestMode(int t) {
  sendCommand(M07+" S" + t);
}

